package com.fs.store.mapper;

import com.fs.course.domain.FsUserWatchCourseStatistics;
import com.fs.course.domain.FsUserWatchStatistics;
import com.fs.course.vo.newfs.FsCourseAnalysisCountVO;
import com.fs.his.vo.OptionsVO;
import com.fs.qw.param.QwFsUserParam;
import com.fs.qw.vo.QwFsUserVO;
import com.fs.store.domain.FsUser;
import com.fs.store.dto.FsUserTransferParamDTO;
import com.fs.store.param.SelectCusListPageParam;
import com.fs.store.param.h5.CourseAnalysisParam;
import com.fs.store.param.h5.FsUserPageListParam;
import com.fs.store.param.h5.UserStatisticsCommonParam;
import com.fs.store.vo.FSUserVO;
import com.fs.store.vo.FsCompanyUserListQueryVO;
import com.fs.store.vo.FsUserTuiVO;
import com.fs.store.vo.h5.*;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;

import java.util.List;
import java.util.Map;

/**
 * 用户Mapper接口
 *
 * @author fs
 * @date 2022-03-15
 */
public interface FsUserMapper
{
    /**
     * 查询用户
     *
     * @param userId 用户ID
     * @return 用户
     */
    public FsUser selectFsUserById(Long userId);

    /**
     * 查询用户列表
     *
     * @param fsUser 用户
     * @return 用户集合
     */
    public List<FsUser> selectFsUserList(FsUser fsUser);

    /**
     * 查询用户列表
     *
     * @param fsUser 用户
     * @return 用户集合
     */
//    @Select({"<script> " +
//            "select  b.total_amount,b.last_buy_time,p.pay_money as number,p.payment_id,p.pay_time," +
//            " u.* FROM fs_user u LEFT JOIN  (" +
//            " SELECT max(payment_id) as payment_id, max(pay_time) as last_buy_time,SUM(pay_money) as total_amount,user_id " +
//            " FROM fs_store_payment" +
//            " WHERE  status = 1 and user_id is not null group by user_id " +
//            ") b ON u.user_id=b.user_id " +
//            " LEFT JOIN fs_store_payment p ON u.user_id=p.user_id AND b.last_buy_time=p.pay_time AND b.payment_id=p.payment_id"+
//            " WHERE 1=1 " +
//            "<if test = 'maps.nickname != null and  maps.nickname !=\"\"    '> " +
//            "and u.nickname like CONCAT('%',#{maps.nickname},'%') " +
//            "</if>" +
//            "<if test = 'maps.phone != null   and  maps.phone !=\"\"   '> " +
//            "and u.phone=#{maps.phone} " +
//            "</if>" +
//            "<if test = 'maps.registerDate != null and  maps.registerDate !=\"\"'> " +
//            "and  DATE_FORMAT(u.register_date, '%Y-%m-%d')  = DATE_FORMAT(#{maps.registerDate}, '%Y-%m-%d') " +
//            "</if>" +
//            "<if test = 'maps.registerCode != null  and  maps.registerCode !=\"\"   '> " +
//            "and u.register_code=#{maps.registerCode} " +
//            "</if>" +
//            "<if test='maps.status != null  and  maps.status !=\"\"'> and u.status = #{maps.status}</if>"+
//            "<if test='maps.level != null  and  maps.level !=\"\"'> and u.level = #{maps.level}</if>"+
//            "<if test='maps.isPromoter != null  and  maps.isPromoter !=\"\"'> and u.is_promoter = #{maps.isPromoter}</if>" +
//            " order by user_id desc "+
//            "</script>"}
//            )
    public List<FSUserVO> selectFsUserVOList(@Param("maps") FsUser fsUser);

    /**
     * 新增用户
     *
     * @param fsUser 用户
     * @return 结果
     */
    public int insertFsUser(FsUser fsUser);

    /**
     * 修改用户
     *
     * @param fsUser 用户
     * @return 结果
     */
    public int updateFsUser(FsUser fsUser);

    /**
     * 删除用户
     *
     * @param userId 用户ID
     * @return 结果
     */
    public int deleteFsUserById(Long userId);

    /**
     * 批量删除用户
     *
     * @param userIds 需要删除的数据ID
     * @return 结果
     */
    public int deleteFsUserByIds(Long[] userIds);

    @Update("update fs_user set integral=integral-#{integral}" +
            " where user_id=#{uid}")
    int decIntegral(@Param("uid") Long uid, @Param("integral")double integral);
    @Update("update fs_user set pay_count=pay_count+1" +
            " where user_id=#{userId}")
    int incPayCount(Long userId);
    @Select("select * from fs_user where phone=#{phone}")
    FsUser selectFsUserByPhone(String phone);


    @Select("select  b.total_amount,b.last_buy_time,p.pay_money as number,p.payment_id,p.pay_time," +
            " u.* FROM fs_user u LEFT JOIN  (" +
            " SELECT max(payment_id) as payment_id, max(pay_time) as last_buy_time,SUM(pay_money) as total_amount,user_id " +
            " FROM fs_store_payment" +
            " WHERE  status = 1 and user_id is not null group by user_id " +
            ") b ON u.user_id=b.user_id " +
            " LEFT JOIN fs_store_payment p ON u.user_id=p.user_id AND b.last_buy_time=p.pay_time AND b.payment_id=p.payment_id"+
            " WHERE u.user_id=#{userId}")
    FSUserVO selectFsUserByUserId(Long userId);


    @Select({"<script> " +
            "select u.*,cu.nick_name as company_user_nick_name  from fs_user u left join company_user cu on u.company_user_id=cu.user_id " +
            "where u.is_del=0 " +
            "<if test = 'maps.nickname != null and  maps.nickname !=\"\"    '> " +
            "and u.nickname like CONCAT('%',#{maps.nickname},'%') " +
            "</if>" +
            "<if test = 'maps.phone != null   and  maps.phone !=\"\"   '> " +
            "and u.phone=#{maps.phone} " +
            "</if>" +
            "<if test = 'maps.companyId != null    '> " +
            "and u.company_id=#{maps.companyId} " +
            "</if>" +
            "<if test = 'maps.status != null   and  maps.status !=\"\"   '> " +
            "and u.status=#{maps.status} " +
            "</if>" +
            "<if test = 'maps.companyUserId != null    '> " +
            "and u.company_user_id=#{maps.companyUserId} " +
            "</if>" +
            " order by u.create_time desc "+
            "</script>"})
    List<FsCompanyUserListQueryVO> selectFsCompanyUserListQuery(@Param("maps")FsUser fsUser);
    @Select({"<script> " +
            "select u.user_id,u.phone,u.avatar,u.nickname,ifnull((select sum(b.number) from fs_user_bill b where b.category='brokerage_price' and b.bill_type=1  and b.tui_user_id=u.user_id and b.user_id=#{userId} ),0)-ifnull((select sum(b.number) from fs_user_bill b where b.category='brokerage_price' and b.bill_type=0 and b.tui_user_id=u.user_id and b.user_id=#{userId}  ),0) as tui_money ,u.create_time from fs_user u " +
            "where u.spread_user_id=#{userId} " +
            "order by tui_money desc " +
            "</script>"})
    List<FsUserTuiVO> selectFsUserTuiList(String userId);
    @Select({"<script> " +
            "select count(1) from fs_user  " +
            "where 1=1 " +
            "<if test = 'type != null and  type ==1  '> " +
            "and  DATE_FORMAT(create_time, '%Y-%m-%d')  = DATE_FORMAT(NOW(), '%Y-%m-%d') " +
            "</if>" +
            "<if test = 'companyId != null  '> " +
            "and  company_id=#{companyId} " +
            "</if>" +
            "</script>"})
    Long selectFsUserCount(@Param("type") int type,@Param("companyId") Long companyId);

    @Select("select * from fs_user where ma_open_id=#{openid}")
    FsUser selectFsUserByMaOpenId(String openid);

    @Select("select * from fs_user where mp_open_id=#{openid}")
    FsUser selectFsUserByMpOpenId(String openid);

    @Select("select * from fs_user where union_id=#{unionId}")
    FsUser selectFsUserByUnionId(String unionId);

    Long selectCrmCustomerCount(int type, Long companyId);

    @Select("select * from fs_user where user_code=#{userCode}")
    FsUser selectFsUserByUserCode(String userCode);
    @Select("select * from fs_user where user_id=#{userId} for update")
    FsUser selectFsUserByIdForUpdate(Long userId);

    @Select({"<script> " +
            "select user_id, nick_name, phone FROM fs_user where is_del=0 "+
            "  <if test=\"nickName != null  and nickName != ''\"> and nick_name like concat('%', #{nickName}, '%')</if>\n" +
            "            <if test=\"phone != null  and phone != ''\"> and phone like concat('%', #{phone}, '%')</if>\n" +
            "order by user_id desc"+
            "</script>"})
    List<QwFsUserVO> selecQwFsUserListVO(QwFsUserParam qwFsUserParam);

    @Select({"<script> " +
            "select user_id, nick_name, phone FROM fs_user where is_del=0 "+
            "  <if test=\"nickName != null  and nickName != ''\"> and nick_name like concat('%', #{nickName}, '%')</if>\n" +
            "            <if test=\"phone != null  and phone != ''\"> and phone =#{phone}</if>\n" +
            "order by user_id desc"+
            "</script>"})
    List<QwFsUserVO> selectQwFsUserListVO(QwFsUserParam qwFsUserParam);


    @Select("select * from fs_user where union_id=#{unionid} limit 1")
    FsUser selectFsUserByUnionid(String unionId);


    List<FsUser> selectFsUserListLimit(FsUser fsUser);

    List<FsUserPageListVO> selectFsUserPageList(FsUserPageListParam param);
    List<FsUserPageListVO> selectFsUserPageListNew(FsUserPageListParam param);
    Long selectFsUserPageListCount(FsUserPageListParam param);

    List<UserListCountVO> getUserNumber(@Param("userId") Long userId, @Param("companyId") Long companyId);

    int getRepeatUserNumber(@Param("userId") Long userId);

    UserDetailsVO getCountWatchCourse (@Param("userId") Long userId, @Param("fsUserId") Long fsUserId, @Param("dateTag") String dateTag);

    UserDetailsVO getCountAnswer (@Param("userId") Long userId, @Param("fsUserId") Long fsUserId, @Param("dateTag") String dateTag);

    UserDetailsVO getCountRedPacket(@Param("userId") Long userId, @Param("fsUserId") Long fsUserId, @Param("dateTag") String dateTag);

    int batchUpdateFsUserByIds(@Param("ids") String[] ids, @Param("status") Integer status);

    /**
     * 查询销售会员总数,今日新增会员数
     * @param companyUserId 销售ID
     * @return map
     */
    Map<String, Long> getUserVipCountByCompanyUserId(@Param("companyUserId") Long companyUserId);

    /**
     * 分页获取客户
     * @param param 参数
     * @return 结果
     */
    List<FsUser> selectCusListPage(SelectCusListPageParam param);
    Long selectCusListPageCount(SelectCusListPageParam param);

    void transferCompanyUser(FsUserTransferParamDTO param);

    FsUserSummaryCountVO countUserSummary(@Param("userId") Long userId, @Param("companyId") Long companyId);

    List<FsUserSummaryCountTagVO> countTag(@Param("userId") Long userId, @Param("companyId") Long companyId);

    Map<String, Long> countUserCourse(UserStatisticsCommonParam param);

    Map<String, Long> countUserAnswer(UserStatisticsCommonParam param);

    Map<String, Object> countUserRedPacket(UserStatisticsCommonParam param);

    Map<String, Long> countCourseDetails(UserStatisticsCommonParam param);

    List<FsUserRankingVO> countUserRankingByComplete(@Param("userId") Long userId, @Param("companyId") Long companyId, @Param("startTime") String startTime, @Param("endTime") String endTime, @Param("periodId")String periodId, @Param("videoId")String videoId, @Param("order")String order);

    List<FsUserRankingVO> countUserRankingByRight(@Param("userId") Long userId, @Param("companyId") Long companyId, @Param("startTime") String startTime, @Param("endTime") String endTime, @Param("periodId")String periodId, @Param("videoId")String videoId, @Param("order")String order);

    List<FsCourseRankingVO> countCourseRankingByComplete(@Param("userId") Long userId, @Param("companyId") Long companyId, @Param("startTime") String startTime, @Param("endTime") String endTime, @Param("periodId")String periodId, @Param("videoId")String videoId, @Param("order")String order);

    List<FsCourseRankingVO> countCourseRankingByRight(@Param("userId") Long userId, @Param("companyId") Long companyId, @Param("startTime") String startTime, @Param("endTime") String endTime, @Param("periodId")String periodId, @Param("videoId")String videoId, @Param("order")String order);

    List<FsCourseAnalysisCountVO> courseAnalysisCourseCount(CourseAnalysisParam param);

    List<FsCourseAnalysisCountVO> courseAnalysisRedPacketCount(CourseAnalysisParam param);

    List<FsCourseAnalysisCountVO> courseAnalysisAnswerCount(CourseAnalysisParam param);

    CompanyUserSummaryCountVO companyUserCount(@Param("companyUserId")String companyUserId, @Param("companyId") Long companyId);

    CompanyUserSummaryCountVO newUserRedPacketCount(@Param("companyUserId")String companyUserId, @Param("companyId") Long companyId);

    List<FsUser> getFsUserByCompanyUserAndId(@Param("companyUserId")Long companyUserId, @Param("userId") Long userId);

    /**
     * 管理端统计看课的相关数据
     * @param param
     * @return
     */
    List<FsCourseAnalysisCountVO> courseAnalysisWatchLog(CourseAnalysisParam param);

    int batchUpdateUserCompanyUser(@Param("userIds") List<Long> userIds, @Param("companyUserId") Long companyUserId, @Param("companyId") Long companyId);

    /**
     * 查询会员选项列表
     * @param params    参数
     * @return  list
     */
    List<OptionsVO> selectUserListByMap(@Param("params") Map<String, Object> params);

    /**
     * 查询重粉用户是否存在
     * @param userIds 会员ids
     * @return
     */
    Integer selectFsUserByUserIds(@Param("userIds") String[] userIds, @Param("companyUserId") Long companyUserId);

    /**
     * 获取会员总数和新增总数-可以按营期/课程
     * @return
     */
    List<FsUserWatchStatistics> selectFsUserTotal();

    List<FsUserWatchCourseStatistics> selectWatchLogCount();

    List<FsUserWatchCourseStatistics> selectRedPacketLogCount();

    List<FsUserWatchCourseStatistics> selectAnswerLogCount();

    List<FsUserWatchCourseStatistics> selectFsUserDetail();

}
